package envoy.client.ui.container;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import envoy.client.data.Settings;
import envoy.client.ui.Theme;
import envoy.client.ui.list.ComponentList;
import envoy.client.ui.list.ComponentList.SelectionMode;
import envoy.client.ui.list.Model;
import envoy.client.ui.list_component.UserComponent;
import envoy.data.Message;
import envoy.data.User;

/**
 * This class defines a dialog to choose contacts from.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ContactsChooserDialog.java</strong><br>
 * Created: <strong>15 Mar 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy Client v0.1-beta
 */
public class ContactsChooserDialog extends JDialog {

	private static final long serialVersionUID = 0L;

	private ComponentList<User>	contactList		= new ComponentList<User>().setModel(new Model<User>())
		.setRenderer((list, user) -> new UserComponent(user));
	private JButton				okButton		= new JButton("Ok");
	private JButton				cancelButton	= new JButton("Cancel");

	private final Theme theme = Settings.getInstance().getCurrentTheme();

	private final JPanel contentPanel = new JPanel();

	/**
	 * Shows a modal contacts-chooser dialog and blocks until the
	 * dialog is hidden. If the user presses the "OK" button, then
	 * this method hides/disposes the dialog and returns the selected element (has
	 * yet
	 * to be casted back to its original type due to the limitations of Generics in
	 * Java).
	 * If the user presses the "Cancel" button or closes the dialog without
	 * pressing "OK", then this method disposes the dialog and returns an empty
	 * <code>ArrayList</code>.
	 *
	 * @param title   the title of the dialog
	 * @param parent  this @{@link Component} will be parsed to
	 *                {@link java.awt.Window#setLocationRelativeTo(Component)} in
	 *                order to change the location of the dialog
	 * @param message the {@link Message} to display on top of the Dialog
	 * @param users   the users that should be displayed
	 * @return the selected Element (yet has to be casted to the wanted type due to
	 *         the Generics limitations in Java)
	 * @since Envoy Client v0.1-beta
	 */
	public static List<User> showForwardingDialog(String title, Component parent, Message message, Collection<User> users) {
		ContactsChooserDialog dialog = new ContactsChooserDialog(parent);
		dialog.setTitle(title);
		dialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		dialog.addCancelButtonActionListener(e -> dialog.dispose());

		List<User> results = new ArrayList<>();
		dialog.addOkButtonActionListener(e -> { results.addAll(dialog.getContactList().getSelectedElements()); dialog.dispose(); });
		Model<User> contactListModel = dialog.getContactList().getModel();
		users.forEach(contactListModel::add);

		dialog.setModalityType(ModalityType.APPLICATION_MODAL);
		dialog.setVisible(true);

		return results;
	}

	/**
	 * @param parent this @{@link Component} will be parsed to
	 *               {@link java.awt.Window#setLocationRelativeTo(Component)}
	 * @since Envoy Client v0.1-beta
	 */
	private ContactsChooserDialog(Component parent) {
		contactList.setSelectionMode(SelectionMode.MULTIPLE);
		contactList.setSelectionHandler((user, comp, isSelected) -> {
			final var theme = Settings.getInstance().getCurrentTheme();
			comp.setBackground(isSelected ? theme.getSelectionColor() : theme.getCellColor());
		});
		setLocationRelativeTo(parent);
		getContentPane().setLayout(new BorderLayout());
		setBackground(theme.getBackgroundColor());
		setForeground(theme.getTextColor());
		setSize(400, 400);
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		contentPanel.add(contactList, BorderLayout.CENTER);
		{
			JPanel buttonPane = new JPanel();
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				okButton = new JButton("OK");
				okButton.setMnemonic(KeyEvent.VK_ENTER);
				okButton.setActionCommand("OK");
				buttonPane.setLayout(new BorderLayout(0, 0));
				buttonPane.add(okButton, BorderLayout.EAST);
				getRootPane().setDefaultButton(okButton);
			}
			{
				cancelButton = new JButton("Cancel");
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton, BorderLayout.WEST);
			}
		}
		applyTheme(Settings.getInstance().getCurrentTheme());
	}

	private void applyTheme(Theme theme) {
		contentPanel.setBackground(theme.getBackgroundColor());
		contentPanel.setForeground(theme.getTextColor());
		contactList.setBackground(theme.getCellColor());
		okButton.setBackground(theme.getInteractableBackgroundColor());
		okButton.setForeground(theme.getTextColor());
		cancelButton.setBackground(theme.getInteractableBackgroundColor());
		cancelButton.setForeground(theme.getTextColor());
	}

	/**
	 * @return the underlying {@link ComponentList}
	 * @since Envoy Client v0.1-beta
	 */
	private ComponentList<User> getContactList() { return contactList; }

	private void addOkButtonActionListener(ActionListener l) { okButton.addActionListener(l); }

	private void addCancelButtonActionListener(ActionListener l) { cancelButton.addActionListener(l); }
}
